part of 'account_bloc.dart';

@immutable
abstract class AccountState extends Equatable {
  const AccountState();
  @override
  List<Object> get props => [];
}

class AccountInitial extends AccountState {
  const AccountInitial();
}

class AccountLoading extends AccountState {
  const AccountLoading();
}

class AccountReady extends AccountState {
  const AccountReady();
}

class AccountLoggedOut extends AccountState {
  const AccountLoggedOut();
}

class AccountLoggedIn extends AccountState {
  const AccountLoggedIn();
}

class AccountError extends AccountState {
  final String message;
  const AccountError({required this.message});

  @override
  List<Object> get props => [message];
}
